#include "main.h"
#include <Arduino.h>
#include <Wire.h>
#include "setup_server.h"
#include <ESP8266WiFi.h>
#include <string>

void setup() 
{
  Serial.begin(9600);
  Serial.println("\nSTART");
  Serial.print("Last Shutdown: ");
  Serial.println(ESP.getResetReason());

  // put your setup code here, to run once:
  screen.begin();
  Wire.begin();
  bme.begin();
  environmentUpdate.attach_scheduled(1, UpdateEnvironment);
  environmentPost.attach_scheduled(60, PostEnvironment);
  InitializeWifi();
}

void loop() 
{
  // put your main code here, to run repeatedly:
  screen.firstPage();
  do{
    screen.setFont(FONT);
    screen.drawStr(2, LINE_1, "Hello World!");
    screen.drawStr(2, LINE_2, "Hi Butlersaurus!");
  }while(screen.nextPage());

  delay(1000);

  screen.firstPage();
  do{
    screen.setFont(FONT);
    screen.drawStr(2, LINE_1, TEMP);
    screen.setCursor(screen.getStrWidth(TEMP), LINE_1);
    screen.print(temp);
    screen.drawStr(2, LINE_2, HUMID);
    screen.setCursor(screen.getStrWidth(HUMID), LINE_2);
    screen.print(humidity);
  }while(screen.nextPage());
  delay(1000);
}

void UpdateEnvironment()
{
    bme.read(pressure, temp, humidity,tempUnit, presUnit);
}

void PostEnvironment()
{
  utils::debug_print("Post");
  requests.begin(client, "http://192.168.64.239/api/environment");
  requests.addHeader("Content-Type", "application/json");
  char dataBuffer[128];
  char formatString[] = "{\"h\":\"%f\",\"t\":\"%f\"}";
  snprintf(&dataBuffer[0], 128, &formatString[0], humidity, temp);
  auto code = requests.POST(dataBuffer);
  if (code > 0){
  utils::debug_print(code);
  }else{
      utils::debug_print(requests.errorToString(code));
  }
  requests.end();
  utils::debug_print("Posted");
}

void InitializeWifi()
{
  WiFi.mode(WIFI_STA);
  WiFi.begin();
  auto ret = WiFi.waitForConnectResult();

  if (ret != WL_CONNECTED)
  {
    utils::debug_print("Could not connect to Wifi");
    auto setup = setup_server();
    std::string ssid = std::string();
    std::string password = std::string();
    setup.get_connection(ssid, password);
  }
  WiFi.setAutoConnect(true);
  utils::debug_print("Wifi Connected: " + WiFi.SSID());
}
